# Population
# ==========

# Loading screen
from platform import python_version
if python_version()[0] != '3':
    print('Please use Python 3.')

import pygame
import os

pygame.init()

WIDTH = 904
HEIGHT = 680
DATAPATH = 'data' + os.path.sep
FONTNAME = DATAPATH + 'BonteDivine.otf'
FONTNAME2 = DATAPATH + 'FreeSansBold.ttf'

def GetCentreBlit(iwidth):
    return (WIDTH - iwidth) // 2

FONTFL = pygame.font.Font(FONTNAME, 80)
LOADINGIMAGE = FONTFL.render('Loading...', 1, (40, 40, 40))
S = pygame.display.set_mode((WIDTH, HEIGHT))
S.fill((0, 0, 0))
S.blit(LOADINGIMAGE, (GetCentreBlit(LOADINGIMAGE.get_width()), 30))
pygame.display.update()
pygame.display.set_caption('Population')

pygame.mixer.music.load('data' + os.path.sep + 'Indore.ogg')
pygame.mixer.music.play(-1)

# Main program
from pygame.locals import *
import random
import math
from src import simulate, graph

from flags import *

def PointInsideRect(point, rect):
    '''Determine if a point is inside a rectangle'''
    right = rect[0] + rect[2]
    bottom = rect[1] + rect[3]
    if rect[0] < point[0] < right and rect[1] < point[1] < bottom:
        return True
    return False

FONTFB = pygame.font.Font(FONTNAME, round(55 * FONTSCALE))
FONTFT = pygame.font.Font(FONTNAME, round(120 * FONTSCALE))
FONTFK = pygame.font.Font(FONTNAME2, round(18 * FONTSCALE))

HORSENAMES = ('Conquest', 'War', 'Famine', 'Death')
STATS = ('Population', 'Food', 'Army')
STATCOLS = ((20, 160, 40),
            (210, 180, 50),
            (200, 30, 10),
            )

HORSES = dict()
HORSETEXTS = dict()
TW = dict()
DESCRIPTIONS = dict()

OP = {'Conquest': ('↓', '↓', '↓'),
      'War': ('↓↓', '↓', '↑↑'),
      'Famine': ('', '↓↓', ''),
      'Death': ('↓↓', '', '↓'),
      }

for horse in HORSENAMES:
    HORSES[horse] = pygame.image.load(DATAPATH + horse + '.png').convert()
    newtext = FONTFB.render(horse, 1, (120, 120, 120))
    HORSETEXTS[horse] = newtext
    TW[horse] = newtext.get_width()
    description = pygame.Surface((150, 100))
    description.fill((0, 0, 0, 0))
    for stat in range(0, len(STATS), 1):
        arrows = FONTFK.render(OP[horse][stat], 1, STATCOLS[stat])
        description.blit(arrows, (30 - arrows.get_width(), stat * 33))
        description.blit(FONTFK.render(STATS[stat], 1, STATCOLS[stat]), (40, stat * 33))
    DESCRIPTIONS[horse] = description

clock = pygame.time.Clock()

TITLE = FONTFT.render('Population', 1, (20, 160, 40))
CBT = (GetCentreBlit(TITLE.get_width()), 30)

# Render buttons
PLAYBUTTON = FONTFB.render('Play', 1, (40, 40, 40))
EXITBUTTON = FONTFB.render('Exit', 1, (40, 40, 40))

# Selected
PLAYBUTTONS = FONTFB.render('Play', 1, (120, 120, 120))
EXITBUTTONS = FONTFB.render('Exit', 1, (120, 120, 120))

WONTEXT = FONTFB.render('You are victorious!', 1, (120, 120, 120))
WTW = WONTEXT.get_width()
LOSTTEXT = FONTFB.render('The Population dies out.', 1, (120, 120, 120))
LTW = LOSTTEXT.get_width()

KEY = pygame.Surface((150, 100))
KEY.fill((0, 0, 0, 0))
KEY.blit(FONTFK.render('— Population', 1, (20, 160, 40)), (0, 0))
KEY.blit(FONTFK.render('— Food', 1, (210, 180, 50)), (0, 33))
KEY.blit(FONTFK.render('— Army', 1, (200, 30, 10)), (0, 66))

sel = 0
ext = False
while not ext:
    for event in pygame.event.get():
        if event.type == QUIT:
            ext = True
            break

        elif event.type == KEYDOWN:
            if event.key == K_ESCAPE:
                ext = True
                break
            elif event.key == K_DOWN:
                sel += 1
            elif event.key == K_UP:
                sel -= 1
            else:
                if sel == 0:
                    # Play the game
                    town = simulate.town([30, 30, 30])
                    gametime = 0
                    lgt = 0
                    gameend = False
                    while not gameend:
                        gametime += 1
                        for event in pygame.event.get():
                            if event.type == QUIT:
                                ext = True
                                gameend = True
                                break
                            elif event.type == KEYDOWN:
                                if event.key == K_ESCAPE:
                                    gameend = True
                                    break

                        if all((gametime > 10, not random.randint(0, round(25 - math.log(gametime ** 1.65) - ((gametime - lgt) ** 0.6))), (gametime - lgt) > 2)):
                            lgt = gametime
                            choices = random.sample(HORSENAMES, 2)
                            S.blit(HORSES[choices[0]], (217, 467))
                            S.blit(HORSES[choices[1]], (479, 467))

                            S.blit(HORSETEXTS[choices[0]], (207 - TW[choices[0]], 467))
                            S.blit(HORSETEXTS[choices[1]], (692, 467))

                            S.blit(DESCRIPTIONS[choices[0]], (58, 550))
                            S.blit(DESCRIPTIONS[choices[1]], (730, 550))

                            pygame.draw.rect(S, (100, 60, 40), (215, 465, 204, 204), 4)
                            pygame.draw.rect(S, (100, 60, 40), (477, 465, 204, 204), 4)

                            pygame.display.update()
                            chosen = False
                            while not chosen:
                                for event in pygame.event.get():
                                    if event.type == MOUSEBUTTONDOWN:
                                        mp = pygame.mouse.get_pos()
                                        if PointInsideRect(mp, (217, 467, 200, 200)):
                                            if choices[0] == 'Conquest':
                                                town.conquest()
                                            elif choices[0] == 'War':
                                                town.war()
                                            elif choices[0] == 'Famine':
                                                town.famine()
                                            elif choices[0] == 'Death':
                                                town.death()
                                            chosen = True

                                        elif PointInsideRect(mp, (479, 467, 200, 200)):
                                            if choices[1] == 'Conquest':
                                                town.conquest()
                                            elif choices[1] == 'War':
                                                town.war()
                                            elif choices[1] == 'Famine':
                                                town.famine()
                                            elif choices[1] == 'Death':
                                                town.death()
                                            chosen = True

                                    elif event.type == QUIT:
                                        ext = True
                                        gameend = True
                                        chosen = True
                                        break
                                    elif event.type == KEYDOWN and event.key == K_ESCAPE:
                                        ext = True
                                        gameend = True
                                        break

                        result = town.do_generation()
                        town.record()
                        g = graph.LineGraph(town.get_records())
                        S.fill((0, 0, 0))
                        S.blit(g, (4, 4))
                        S.blit(KEY, (20, 20))
                        pygame.draw.rect(S, (100, 60, 40), (2, 2, 900, 454), 4)
                        if result == 'Lost':
                            back = False
                            while not back:
                                for event in pygame.event.get():
                                    if event.type in (KEYDOWN, MOUSEBUTTONDOWN):
                                        back = True
                                        gameend = True
                                        break
                                S.blit(g, (4, 4))
                                S.blit(LOSTTEXT, ((WIDTH - LTW) / 2, 550))
                                S.blit(KEY, (20, 20))
                                pygame.display.update()

                        elif result == 'Won':
                            back = False
                            while not back:
                                for event in pygame.event.get():
                                    if event.type in (KEYDOWN, MOUSEBUTTONDOWN):
                                        back = True
                                        gameend = True
                                        break
                                S.blit(g, (4, 4))
                                S.blit(WONTEXT, ((WIDTH - WTW) / 2, 550))
                                S.blit(KEY, (20, 20))
                                pygame.display.update()
                        pygame.display.update()
                        clock.tick(SPEED)

                elif sel == 1:
                    ext = True
                    break

            sel = sel % 2

    S.fill((0, 0, 0))

    S.blit(TITLE, CBT)
    # Draw buttons
    if sel == 0:
        S.blit(PLAYBUTTONS, (30, 160))
        S.blit(EXITBUTTON, (30, 230))
    elif sel == 1:
        S.blit(PLAYBUTTON, (30, 160))
        S.blit(EXITBUTTONS, (30, 230))

    pygame.display.update()
pygame.quit()
